<?php

namespace Drupal\slider_pro\Plugin\views\style;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\style\StylePluginBase;
use Drupal\views\ResultRow;

/**
 * @ingroup views_style_plugins
 *
 * @ViewsStyle(
 *   id = "slider_pro",
 *   title = @Translation("Slider Pro"),
 *   help = @Translation("Displays a view as a Slider Pro, using the Slider Pro jQuery plugin."),
 *   theme = "slider_pro_views_style",
 *   theme_file = "slider_pro_views.theme.inc",
 *   display_types = {"normal"}
 * )
 */
class SliderPro extends StylePluginBase {
  /**
   * {@inheritdoc}
   */
  protected $usesRowPlugin = FALSE;

  /**
   * {@inheritdoc}
   */
  protected $usesFields = TRUE;

  /**
   * {@inheritdoc}
   */
  protected $usesOptions = TRUE;

  /**
   * {@inheritdoc}
   */
  protected $usesGrouping = FALSE;

  /**
   * {@inheritdoc}
   */
  protected $usesRowClass = FALSE;

  /**
   * {@inheritdoc}
   */
  public function evenEmpty() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $fields = $this->getAvailableFields();
    if (empty($fields)) {
      drupal_set_message($this->t('To configure Slider Pro you have to add at least one field'), 'error');
      return $form;
    }

    $general_options = isset($this->options['general']) ? $this->options['general'] : [
      'width' => '100%',
      'visible_size' => 'auto',
      'force_size' => 'none',
      'height' => '200px',
      'orientation' => 'horizontal',
      'buttons' => 0,
      'transition' => 0,
      'arrows' => 0,
      'fields' => []
    ];
    $thumb_options = isset($this->options['thumbnails']) ? $this->options['thumbnails'] : [
      'position' => 0,
      'fields' => array_keys($this->getAvailableFields()),
      'width' => '100',
      'height' => '80',
      'thumbnail_pointer' => 0,
      'thumbnail_arrows' => 0,
      'fade_thumbnail_arrows' => 1,
    ];
    $layer_options = isset($this->options['layers_wrapper']) ? $this->options['layers_wrapper'] : [
      'wait_for_layers' => 0,
      'auto_scale_layers' => 1,
      'number_of_layers' => 0,
      'layers' => [],
    ];
    $advanced_options = isset($this->options['advanced']) ? $this->options['advanced'] : [
      'center_image' => 1,
      'allow_scale_up' => 1,
      'auto_height' => 0,
      'start_slide' => 0,
      'shuffle' => 0,
      'loop' => 1,
      'autoplay' => 1,
      'autoplay_delay' => 5000,
      'autoplay_direction' => 'normal',
      'autoplay_on_hover' => 'pause',
      'slide_distance' => 10,
      'keyboard' => 1,
      'full_screen' => 0,
      'fade_full_screen' => 1,
      'update_hash' => 0,
    ];

    $form['general'] = [
      '#type' => 'details',
      '#title' => $this->t('General settings'),
      '#open' => TRUE,
    ];

    $form['general']['width'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Width'),
      '#description' => 'The width of the slider. Eg 200px or 50%',
      '#default_value' => $general_options['width'],
      '#required' => TRUE,
    ];

    $form['general']['visible_size'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Visible size'),
      '#description' => 'Sets the size (for example 100%) of the visible area, allowing for more slides to become visible near the selected slide. Be sure that the width of your slider is less than 100%. Example',
      '#default_value' => $general_options['visible_size'],
      '#required' => TRUE,
    ];

    $form['general']['force_size'] = [
      '#type' => 'select',
      '#title' => $this->t('Force size'),
      '#options' => [
        'none' => $this->t('None'),
        'fullWidth' => $this->t('Full width'),
        'fullWindow' => $this->t('Full window'),
      ],
      '#default_value' => $general_options['force_size'],
      '#description' => $this->t('Indicates if the size of the slider will be forced to full width or full window.'),
      '#required' => TRUE,
    ];

    $form['general']['height'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Height'),
      '#default_value' => $general_options['height'],
      '#description' => 'The height of the slider. Eg 200px or 50%',
    ];

    $form['general']['orientation'] = [
      '#type' => 'select',
      '#title' => $this->t('Orientation'),
      '#options' => [
        'horizontal' => $this->t('Horizontal'),
        'vertical' => $this->t('Vertical'),
      ],
      '#default_value' => $general_options['orientation'],
      '#description' => $this->t('Indicates whether the slides will be arranged horizontally or vertically.'),
      '#required' => TRUE,
    ];

    $form['general']['buttons'] = [
      '#type' => 'select',
      '#title' => $this->t('Buttons'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $general_options['buttons'],
      '#description' => $this->t('Indicates whether the buttons will be created.'),
      '#required' => TRUE,
    ];

    $form['general']['transition'] = [
      '#type' => 'select',
      '#title' => $this->t('Transition'),
      '#options' => [
        0 => $this->t('Slide'),
        1 => $this->t('Fade'),
      ],
      '#default_value' => $general_options['transition'],
      '#description' => $this->t('Indicates which transition will be used.'),
      '#required' => TRUE,
    ];

    $form['general']['arrows'] = [
      '#type' => 'select',
      '#title' => $this->t('Arrows'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $general_options['arrows'],
      '#description' => $this->t('Indicates whether the arrow buttons will be created.'),
      '#required' => TRUE,
    ];

    $form['general']['fields'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Fields'),
      '#options' => $this->getAvailableFields(),
      '#default_value' => $general_options['fields'],
      '#description' => $this->t('Select which fields you want to display on the slider.'),
      '#required' => TRUE,
    ];

    $form['thumbnails'] = [
      '#type' => 'details',
      '#title' => $this->t('Thumbnail settings'),
      '#open' => ($thumb_options['position']) ? TRUE : FALSE,
    ];

    $form['thumbnails']['position'] = [
      '#type' => 'select',
      '#title' => $this->t('Position'),
      '#options' => [
        0 => $this->t('No thumbs'),
        'left' => $this->t('Left'),
        'right' => $this->t('Right'),
        'top' => $this->t('Top'),
        'bottom' => $this->t('Bottom'),
      ],
      '#default_value' => $thumb_options['position'],
      '#required' => TRUE,
      '#description' => $this->t('Sets the position of the thumbnail scroller.'),
    ];

    $form['thumbnails']['fields'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Fields'),
      '#options' => $this->getAvailableFields(),
      '#description' => $this->t('Select which fields you want to display on the thumbs.'),
      '#default_value' => $thumb_options['fields'],
      '#required' => TRUE,
      '#states' => [
        'invisible' => [
          'select[name="style_options[thumbnails][position]"]' => ['value' => '0'],
        ],
      ],
    ];

    $form['thumbnails']['width'] = [
      '#type' => 'number',
      '#title' => $this->t('Width'),
      '#description' => $this->t('The width of each thumbnail. Eg 200px.'),
      '#default_value' => $thumb_options['width'],
      '#required' => TRUE,
      '#states' => [
        'invisible' => [
          'select[name="style_options[thumbnails][position]"]' => ['value' => '0'],
        ],
      ],
    ];

    $form['thumbnails']['height'] = [
      '#type' => 'number',
      '#title' => $this->t('height'),
      '#description' => $this->t('The height of each thumbnail. Eg 200px.'),
      '#default_value' => $thumb_options['height'],
      '#required' => TRUE,
      '#states' => [
        'invisible' => [
          'select[name="style_options[thumbnails][position]"]' => ['value' => '0'],
        ],
      ],
    ];

    $form['thumbnails']['thumbnail_pointer'] = [
      '#type' => 'select',
      '#title' => $this->t('Pointer'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $thumb_options['thumbnail_pointer'],
      '#required' => TRUE,
      '#description' => $this->t('Indicates if a pointer will be displayed for the selected thumbnail.'),
      '#states' => [
        'invisible' => [
          'select[name="style_options[thumbnails][position]"]' => ['value' => '0'],
        ],
      ],
    ];

    $form['thumbnails']['thumbnail_arrows'] = [
      '#type' => 'select',
      '#title' => $this->t('Arrows'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $thumb_options['thumbnail_arrows'],
      '#required' => TRUE,
      '#description' => $this->t('Indicates whether the thumbnail arrows will be enabled.'),
      '#states' => [
        'invisible' => [
          'select[name="style_options[thumbnails][position]"]' => ['value' => '0'],
        ],
      ],
    ];

    $form['thumbnails']['fade_thumbnail_arrows'] = [
      '#type' => 'select',
      '#title' => $this->t('Fade arrows'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $thumb_options['fade_thumbnail_arrows'],
      '#required' => TRUE,
      '#description' => $this->t('Indicates whether the thumbnail arrows will be faded.'),
      '#states' => [
        'invisible' => [
          [
            'select[name="style_options[thumbnails][position]"]' => ['value' => '0'],
          ],
          [
            'select[name="style_options[thumbnails][thumbnail_arrows]"]' => ['value' => '0'],
          ],
        ],
      ],
    ];

    $form['layers_wrapper'] = [
      '#type' => 'details',
      '#title' => $this->t('Layer settings'),
      '#open' => !empty($layer_options['number_of_layers']) ? TRUE : FALSE,
    ];

    $form['layers_wrapper']['number_of_layers'] = [
      '#type' => 'select',
      '#title' => $this->t('Number of layers'),
      '#options' => array_combine(
        [0, 1, 2, 3, 4, 5],
        [$this->t('None'), 1, 2, 3, 4, 5]
      ),
      '#default_value' => $layer_options['number_of_layers'],
      '#description' => $this->t('Provide the number of layers you want to use. Afterwards save the options and re-open them to start configuring your layers.'),
    ];

    $form['layers_wrapper']['wait_for_layers'] = [
      '#type' => 'select',
      '#title' => $this->t('Wait for layers'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $layer_options['wait_for_layers'],
      '#description' => $this->t('Indicates whether the slider will wait for the layers to disappear before going to a new slide.'),
    ];

    $form['layers_wrapper']['auto_scale_layers'] = [
      '#type' => 'select',
      '#title' => $this->t('Auto scale layers'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $layer_options['auto_scale_layers'],
      '#description' => $this->t('Indicates whether the layers will be scaled automatically.'),
    ];

    $form['layers_wrapper']['layers'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Fields'),
        $this->t('Background'),
        $this->t('Position'),
        $this->t('Show transition'),
        $this->t('Hide transition'),
        $this->t('Delay'),
        $this->t('Duration'),
        $this->t('Weight'),
      ],
      '#attributes' => [
        'id' => 'slider-pro-layers',
      ],
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'weight',
        ],
      ],
      '#empty' => $this->t('No layers configured yet...'),
    ];

    for ($key = 0; $key < $layer_options['number_of_layers']; $key++) {
      $layer = isset($layer_options['layers'][$key]) ? $layer_options['layers'][$key] : [
        'fields' => [],
        'background' => NULL,
        'position' => NULL,
        'show_transition' => NULL,
        'hide_transition' => NULL,
        'show_delay' => NULL,
        'stay_duration' => NULL,
        'weight' => 0,
      ];

      $form['layers_wrapper']['layers'][$key]['#attributes']['class'][] = 'draggable';
      $form['layers_wrapper']['layers'][$key]['#weight'] = $layer['weight'];
      $form['layers_wrapper']['layers'][$key]['fields'] = [
        '#type' => 'checkboxes',
        '#title' => $this->t('Fields'),
        '#title_display' => 'invisible',
        '#options' => $this->getAvailableFields(),
        '#default_value' => $layer['fields'],
        '#description' => $this->t('Select which fields you want to display on this layer.'),
        '#required' => TRUE,
      ];

      $form['layers_wrapper']['layers'][$key]['background'] = [
        '#type' => 'select',
        '#title' => $this->t('Background'),
        '#title_display' => 'invisible',
        '#options' => [
          '' => $this->t('None'),
          'sp-white' => $this->t('White transparant'),
          'sp-black' => $this->t('Black transparant'),
        ],
        '#default_value' => $layer['background'],
        '#required' => TRUE,
      ];

      $form['layers_wrapper']['layers'][$key]['position'] = [
        '#type' => 'select',
        '#title' => $this->t('Position'),
        '#title_display' => 'invisible',
        '#options' => [
          'topLeft' => $this->t('Top left'),
          'topCenter' => $this->t('Top center'),
          'topRight' => $this->t('Top right'),
          'bottomLeft' => $this->t('Bottom left'),
          'bottomCenter' => $this->t('Bottom center'),
          'bottomRight' => $this->t('Bottom right'),
          'centerLeft' => $this->t('Center left'),
          'centerRight' => $this->t('Center right'),
          'centerCenter' => $this->t('Center center'),
        ],
        '#default_value' => $layer['position'],
        '#required' => TRUE,
      ];

      $form['layers_wrapper']['layers'][$key]['show_transition'] = [
        '#type' => 'select',
        '#title' => $this->t('Show transition'),
        '#title_display' => 'invisible',
        '#options' => [
          'left' => $this->t('Left'),
          'right' => $this->t('Right'),
          'up' => $this->t('Up'),
          'down' => $this->t('Down'),
        ],
        '#default_value' => $layer['show_transition'],
        '#required' => TRUE,
      ];

      $form['layers_wrapper']['layers'][$key]['hide_transition'] = [
        '#type' => 'select',
        '#title' => $this->t('Show transition'),
        '#title_display' => 'invisible',
        '#options' => [
          'left' => $this->t('Left'),
          'right' => $this->t('Right'),
          'up' => $this->t('Up'),
          'down' => $this->t('Down'),
        ],
        '#default_value' => $layer['hide_transition'],
        '#required' => TRUE,
      ];

      $form['layers_wrapper']['layers'][$key]['show_delay'] = [
        '#type' => 'number',
        '#title' => $this->t('Show delay'),
        '#title_display' => 'invisible',
        '#size' => 5,
        '#required' => TRUE,
        '#default_value' => $layer['show_delay'],
        '#description' => $this->t('Sets a delay for the show transition. This delay starts from the moment when the transition to the new slide starts.'),
      ];

      $form['layers_wrapper']['layers'][$key]['stay_duration'] = [
        '#type' => 'number',
        '#title' => $this->t('Stay duration'),
        '#title_display' => 'invisible',
        '#size' => 5,
        '#required' => TRUE,
        '#default_value' => $layer['stay_duration'],
        '#description' => $this->t('Sets how much time a layer will stay visible before being hidden automatically.'),
      ];

      // @todo: use this button if figured out how ajax calls in this form work.
      /*$form['layers_wrapper']['layers'][$key]['remove'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove'),
        '#submit' => [[$this, 'removeLayerSubmit']],
        '#ajax' => [
          'callback' => [
            'Drupal\slider_pro\Plugin\views\style\SliderPro',
            'ajaxRefreshLayers'
          ],
        ],
        '#limit_validation_errors' => [],
      ];*/

      $form['layers_wrapper']['layers'][$key]['weight'] = [
        '#type' => 'weight',
        '#title' => $this->t('Weight'),
        '#title_display' => 'invisible',
        '#default_value' => $layer['weight'],
        '#attributes' => [
          'class' => ['weight'],
        ],
      ];
    }

    //Sort by weight.
    uasort($form['layers_wrapper']['layers'], [
      '\Drupal\Component\Utility\SortArray',
      'sortByWeightProperty'
    ]);


    // @todo: use this button if figured out how ajax calls in this form work.
    /*$form['layers_wrapper']['add_layer'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add layer'),
      '#submit' => [[$this, 'addNewLayerSubmit']],
      '#ajax' => [
        'callback' => [
          'Drupal\slider_pro\Plugin\views\style\SliderPro',
          'ajaxRefreshLayers'
        ],
      ],
      '#limit_validation_errors' => [],
    ];*/

    $form['advanced'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced settings'),
      '#open' => FALSE,
    ];

    $form['advanced']['center_image'] = [
      '#type' => 'select',
      '#title' => $this->t('Center image'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $advanced_options['center_image'],
      '#description' => $this->t('Indicates if the image will be centered.'),
      '#required' => TRUE,
    ];

    $form['advanced']['allow_scale_up'] = [
      '#type' => 'select',
      '#title' => $this->t('Allow scale up'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $advanced_options['allow_scale_up'],
      '#description' => $this->t('Indicates if the image can be scaled up more than its original size.'),
      '#required' => TRUE,
    ];

    $form['advanced']['auto_height'] = [
      '#type' => 'select',
      '#title' => $this->t('Auto height'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $advanced_options['auto_height'],
      '#description' => $this->t('Indicates if height of the slider will be adjusted to the height of the selected slide.'),
      '#required' => TRUE,
    ];

    $form['advanced']['slide_distance'] = [
      '#type' => 'number',
      '#title' => $this->t('Slide distance'),
      '#default_value' => $advanced_options['slide_distance'],
      '#description' => $this->t('Sets the distance between the slides.'),
      '#required' => TRUE,
    ];

    $form['advanced']['start_slide'] = [
      '#type' => 'number',
      '#title' => $this->t('Start slide'),
      '#default_value' => $advanced_options['start_slide'],
      '#description' => $this->t('Sets the slide that will be selected when the slider loads.'),
      '#required' => TRUE,
    ];

    $form['advanced']['shuffle'] = [
      '#type' => 'select',
      '#title' => $this->t('Shuffle'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $advanced_options['shuffle'],
      '#description' => $this->t('Indicates if the slides will be shuffled.'),
      '#required' => TRUE,
    ];

    $form['advanced']['loop'] = [
      '#type' => 'select',
      '#title' => $this->t('Loop'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $advanced_options['loop'],
      '#description' => $this->t('Indicates if the slider will be loopable (infinite scrolling).'),
      '#required' => TRUE,
    ];

    $form['advanced']['autoplay'] = [
      '#type' => 'select',
      '#title' => $this->t('Autoplay'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $advanced_options['autoplay'],
      '#description' => $this->t('Indicates whether or not autoplay will be enabled.'),
      '#required' => TRUE,
    ];

    $form['advanced']['autoplay_delay'] = [
      '#type' => 'number',
      '#title' => $this->t('Autoplay delay'),
      '#default_value' => $advanced_options['autoplay_delay'],
      '#description' => $this->t('Sets the delay/interval (in milliseconds) at which the autoplay will run.'),
      '#required' => TRUE,
      '#states' => [
        'visible' => [
          'select[name="style_options[advanced][autoplay]"]' => ['value' => '1'],
        ],
      ],
    ];

    $form['advanced']['autoplay_direction'] = [
      '#type' => 'select',
      '#title' => $this->t('Autoplay direction'),
      '#options' => [
        'normal' => $this->t('Normal'),
        'backwards' => $this->t('Backwards'),
      ],
      '#default_value' => $advanced_options['autoplay_direction'],
      '#description' => $this->t('Indicates whether autoplay will navigate to the next slide or previous slide.'),
      '#required' => TRUE,
      '#states' => [
        'visible' => [
          'select[name="style_options[advanced][autoplay]"]' => ['value' => '1'],
        ],
      ],
    ];

    $form['advanced']['autoplay_on_hover'] = [
      '#type' => 'select',
      '#title' => $this->t('Autoplay on hover'),
      '#options' => [
        'pause' => $this->t('Pause'),
        'stop' => $this->t('Stop'),
        'none' => $this->t('None'),
      ],
      '#default_value' => $advanced_options['autoplay_on_hover'],
      '#description' => $this->t('Indicates if the autoplay will be paused or stopped when the slider is hovered.'),
      '#required' => TRUE,
      '#states' => [
        'visible' => [
          'select[name="style_options[advanced][autoplay]"]' => ['value' => '1'],
        ],
      ],
    ];

    $form['advanced']['keyboard'] = [
      '#type' => 'select',
      '#title' => $this->t('Keyboard'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $advanced_options['keyboard'],
      '#description' => $this->t('Indicates whether keyboard navigation will be enabled.'),
      '#required' => TRUE,
    ];

    $form['advanced']['full_screen'] = [
      '#type' => 'select',
      '#title' => $this->t('Full screen'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $advanced_options['full_screen'],
      '#description' => $this->t('Indicates whether the full-screen button is enabled.'),
      '#required' => TRUE,
    ];

    $form['advanced']['fade_full_screen'] = [
      '#type' => 'select',
      '#title' => $this->t('Fade full screen'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $advanced_options['fade_full_screen'],
      '#description' => $this->t('Indicates whether the button will fade in only on hover.'),
      '#required' => TRUE,
      '#states' => [
        'visible' => [
          'select[name="style_options[advanced][full_screen]"]' => ['value' => '1'],
        ],
      ],
    ];

    $form['advanced']['update_hash'] = [
      '#type' => 'select',
      '#title' => $this->t('Update hash'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $advanced_options['update_hash'],
      '#description' => $this->t('Indicates whether the hash will be updated when a new slide is selected.'),
      '#required' => TRUE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $rows = [];
    $fields = array_keys($this->getAvailableFields());

    if (empty($this->options['general'])) {
      // Settings have not been set yet.
      return $rows;
    }

    $general_options = $this->options['general'];
    $thumb_options = $this->options['thumbnails'];
    $layer_options = $this->options['layers_wrapper'];

    for ($i = 0; $i < count($this->view->result); $i++) {
      $rows[$i]['layers'] = [];
      for ($j = 0; $j < count($fields); $j++) {
        $field = $fields[$j];
        $rendered_field = $this->view->style_plugin->getField($i, $field);
        if (in_array($field, array_filter($general_options['fields']))) {
          $rows[$i]['slide'][$fields[$j]] = $rendered_field;
        }
        if (in_array($field, array_filter($thumb_options['fields']))) {
          $rows[$i]['thumb'][$fields[$j]] = $rendered_field;
        }

        if (!$layer_options['layers']) {
          // Make sure layers is an array.
          $layer_options['layers'] = [];
        }
        $layers = array_slice($layer_options['layers'], 0, $layer_options['number_of_layers']);
        foreach ($layers as $key => $layer) {
          if (!isset($rows[$i]['layers'][$key])) {
            $rows[$i]['layers'][$key] = $layer;
          }
          if (in_array($field, array_filter($layer['fields']))) {
            $rows[$i]['layers'][$key]['content'][$fields[$j]] = $rendered_field;
          }
        }
      }
    }

    // Unset fields from all layers. Don't need it while rendering.
    foreach ($rows as &$row) {
      foreach ($row['layers'] as &$layer) {
        unset($layer['fields']);
        unset($layer['weight']);
        unset($layer['remove']);
      }
    }

    $build = array(
      '#theme' => $this->themeFunctions(),
      '#view' => $this->view,
      '#options' => $this->options,
      '#rows' => $rows,
    );

    return $build;
  }

  /**
   * Ajax callback to refresh layers.
   * @param $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @return \Drupal\Core\Ajax\AjaxResponse
   */
  public static function ajaxRefreshLayers($form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $response->addCommand(new ReplaceCommand('#slider-pro-layers', $form['options']['style_options']['layers_wrapper']['layers']));
    return $response;
  }

  /**
   * Submit callback to add new layer.
   * @param $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function addNewLayerSubmit($form, FormStateInterface $form_state) {
    $form_state->setRebuild(TRUE);
  }

  /**
   * Submit callback to remove a layer.
   * @param $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function removeLayerSubmit($form, FormStateInterface $form_state) {
    $form_state->setRebuild(TRUE);
  }

  /**
   * Returns option list of fields available on view.
   */
  protected function getAvailableFields() {
    $view = $this->view;
    return $view->display_handler->getFieldLabels();
  }

}
