<?php

namespace Drupal\slider_pro;

/**
 * Class SliderProOptionset
 * @package Drupal\slider_pro
 */
class SliderProOptionset {

  protected $useThumbs;
  protected $hasLayers;
  protected $allowFullScreen;
  protected $hasAutoPlay;
  protected $options;

  /**
   * SliderProOptionset constructor.
   * @param $options
   */
  public function __construct($options) {
    $this->options = $options;
    $this->useThumbs = !empty($options['thumbnails']['position']);
    $this->hasLayers = !empty($options['layers_wrapper']['layers']);
    $this->allowFullScreen = !empty($this->options['advanced']['full_screen']);
    $this->hasAutoPlay = !empty($this->options['advanced']['autoplay']);
  }

  /**
   * Returns optionset.
   * @return array|bool
   */
  public function toArray() {

    if (!isset($this->options['general'])) {
      return FALSE;
    }

    $optionset = [
      'width' => $this->options['general']['width'],
      'visibleSize' => $this->options['general']['visible_size'],
      'forceSize' => $this->options['general']['force_size'],
      'height' => $this->options['general']['height'],
      'orientation' => $this->options['general']['orientation'],
      'buttons' => ($this->options['general']['buttons']) ? TRUE : FALSE,
      'fade' => ($this->options['general']['transition']) ? TRUE : FALSE,
      'arrows' => ($this->options['general']['arrows']) ? TRUE : FALSE,
      'centerImage' => ($this->options['advanced']['center_image']) ? TRUE : FALSE,
      'allowScaleUp' => ($this->options['advanced']['allow_scale_up']) ? TRUE : FALSE,
      'autoHeight' => ($this->options['advanced']['auto_height']) ? TRUE : FALSE,
      'startSlide' => (int) $this->options['advanced']['start_slide'],
      'shuffle' => ($this->options['advanced']['shuffle']) ? TRUE : FALSE,
      'loop' => ($this->options['advanced']['loop']) ? TRUE : FALSE,
      'autoplay' => ($this->options['advanced']['autoplay']) ? TRUE : FALSE,
      'slideDistance' => (int) $this->options['advanced']['autoplay'],
      'keyboard' => ($this->options['advanced']['keyboard']) ? TRUE : FALSE,
      'fullScreen' => ($this->options['advanced']['full_screen']) ? TRUE : FALSE,
      'updateHash' => ($this->options['advanced']['update_hash']) ? TRUE : FALSE,
    ];

    if ($this->hasAutoPlay) {
      $optionset['autoplayDelay'] = (int) $this->options['advanced']['autoplay_delay'];
      $optionset['autoplayDirection'] = $this->options['advanced']['autoplay_direction'];
      $optionset['autoplayOnHover'] = $this->options['advanced']['autoplay_on_hover'];
    }

    if ($this->allowFullScreen) {
      $optionset['fadeFullScreen'] = ($this->options['advanced']['fade_full_screen']) ? TRUE : FALSE;
    }

    if ($this->useThumbs) {
      $optionset['thumbnailWidth'] = (int) $this->options['thumbnails']['width'];
      $optionset['thumbnailHeight'] = (int) $this->options['thumbnails']['height'];
      $optionset['thumbnailsPosition'] = $this->options['thumbnails']['position'];
      $optionset['thumbnailPointer'] = ($this->options['thumbnails']['thumbnail_pointer']) ? TRUE : FALSE;
      $optionset['thumbnailArrows'] = ($this->options['thumbnails']['thumbnail_arrows']) ? TRUE : FALSE;
      if ($optionset['thumbnailArrows']) {
        $optionset['fadeThumbnailArrows'] = ($this->options['thumbnails']['fade_thumbnail_arrows']) ? TRUE : FALSE;
      }
    }

    if ($this->hasLayers) {
      $optionset['waitForLayers'] = ($this->options['layers_wrapper']['wait_for_layers']) ? TRUE : FALSE;
      $optionset['autoScaleLayers'] = ($this->options['layers_wrapper']['auto_scale_layers']) ? TRUE : FALSE;
    }

    return $optionset;
  }

}